@@ -188,8 +188,7 @@ pango_win32_font_map_class_init (PangoFontMapClass *class)
   class->load_font = pango_win32_font_map_load_font;
   class->list_families = pango_win32_font_map_list_families;
 
-  if (pango_win32_hdc == NULL)
-    pango_win32_hdc = CreateDC ("DISPLAY", NULL, NULL, NULL);
+  pango_win32_get_dc ();
 }
 
 static PangoWin32FontMap *fontmap = NULL;
@@ -246,7 +245,7 @@ pango_win32_font_map_for_display (void)
   if (fontmap != NULL)
     return PANGO_FONT_MAP (fontmap);
 
-  fontmap = (PangoWin32FontMap *) g_type_create_instance (PANGO_TYPE_WIN32_FONT_MAP);
+  fontmap = g_object_new (PANGO_TYPE_WIN32_FONT_MAP, NULL);
   
   fontmap->font_cache = pango_win32_font_cache_new ();
   fontmap->freed_fonts = g_queue_new ();
@@ -271,7 +270,7 @@ pango_win32_shutdown_display (void)
   if (fontmap)
     {
       pango_win32_fontmap_cache_clear (fontmap);
-      g_object_unref (G_OBJECT (fontmap));
+      g_object_unref (fontmap);
 
       fontmap = NULL;
     }
@@ -471,7 +470,7 @@ pango_win32_font_map_load_font (PangoFontMap               *fontmap,
 		  PING (("size matches"));
 		  result = (PangoFont *)win32font;
 
-		  g_object_ref (G_OBJECT (result));
+		  g_object_ref (result);
 		  if (win32font->in_cache)
 		    pango_win32_fontmap_cache_remove (fontmap, win32font);
 		  
@@ -782,7 +781,7 @@ pango_win32_insert_font (PangoWin32FontMap *win32fontmap,
    * same entry at the moment and it isn't crashing on g_free () ???
    * Maybe a memory leak ...
    */
-  switch (lfp->lfPitchAndFamily & (0x0F << 4)) /* bit 4...7 */
+  switch (lfp->lfPitchAndFamily & 0xF0)
     { 
     case FF_MODERN : /* monospace */
       PING(("monospace"));
@@ -1010,10 +1009,10 @@ pango_win32_fontmap_cache_add (PangoFontMap   *fontmap,
   if (win32fontmap->freed_fonts->length == MAX_FREED_FONTS)
     {
       PangoWin32Font *old_font = g_queue_pop_tail (win32fontmap->freed_fonts);
-      g_object_unref (G_OBJECT (old_font));
+      g_object_unref (old_font);
     }
 
-  g_object_ref (G_OBJECT (win32font));
+  g_object_ref (win32font);
   g_queue_push_head (win32fontmap->freed_fonts, win32font);
   win32font->in_cache = TRUE;
 }
@@ -1036,7 +1035,7 @@ pango_win32_fontmap_cache_remove (PangoFontMap   *fontmap,
   win32fontmap->freed_fonts->length--;
   win32font->in_cache = FALSE;
 
-  g_object_unref (G_OBJECT (win32font));
+  g_object_unref (win32font);
 }
 
 static void
