@@ -414,7 +414,7 @@ pango_x_font_new (PangoFontMap *fontmap, const char *spec, int size)
   g_return_val_if_fail (fontmap != NULL, NULL);
   g_return_val_if_fail (spec != NULL, NULL);
 
-  result = (PangoXFont *)g_type_create_instance (PANGO_TYPE_X_FONT);
+  result = g_object_new (PANGO_TYPE_X_FONT, NULL);
   
   result->fontmap = fontmap;
   g_object_ref (fontmap);
@@ -455,6 +455,19 @@ pango_x_load_font (Display    *display,
 
   return (PangoFont *)result;
 }
+
+
+#define FLUSH						\
+  G_STMT_START {					\
+    if (charcount)					\
+      {							\
+	XDrawString16 (display, d, gc,			\
+		       glyph_x0, glyph_y0,		\
+		       xcharbuffer, charcount);		\
+	charcount = 0;					\
+      }							\
+  } G_STMT_END
+
  
 /**
  * pango_x_render:
@@ -477,15 +490,21 @@ pango_x_render  (Display           *display,
 		 int                x, 
 		 int                y)
 {
-  /* Slow initial implementation. For speed, it should really
-   * collect the characters into runs, and draw multiple
-   * characters with each XDrawString16 call.
-   */
   Font old_fid = None;
   XFontStruct *fs;
   int i;
   int x_off = 0;
 
+  /*
+   * We collect the characters in this buffer as long as the font does not
+   * change.  At that time, or when the buffer runs full, or at the end,
+   * then we empty the buffer.
+   */
+  XChar2b xcharbuffer[1000];
+  int glyph_x0 = 0, expected_x = 0; /* x/y initializations are to quiet GCC */
+  int glyph_y0 = 0;
+  int charcount = 0;
+
   g_return_if_fail (display != NULL);
   g_return_if_fail (glyphs != NULL);
   
@@ -515,7 +534,9 @@ pango_x_render  (Display           *display,
 	  int stroke_thick;
 
           gunichar wc;
-          
+
+	  FLUSH;
+
           x1 = glyph_x;
           y1 = glyph_y - PANGO_PIXELS (metrics->ascent);
           x2 = x1 + PANGO_PIXELS (glyphs->glyphs[i].geometry.width);
@@ -624,35 +645,47 @@ pango_x_render  (Display           *display,
 	  guint16 subfont_index = PANGO_X_GLYPH_SUBFONT (glyph);
 	  PangoXSubfontInfo *subfont;
       
-	  XChar2b c;
-	  
 	  subfont = pango_x_find_subfont (font, subfont_index);
 	  if (subfont)
 	    {
-	      c.byte1 = index / 256;
-	      c.byte2 = index % 256;
-	      
 	      fs = pango_x_get_font_struct (font, subfont);
 	      if (!fs)
 		continue;
 	      
 	      if (fs->fid != old_fid)
 		{
+		  FLUSH;
 		  XSetFont (display, gc, fs->fid);
 		  old_fid = fs->fid;
 		}
-	      
-	      XDrawString16 (display, d, gc,
-			     glyph_x, glyph_y,
-			     &c, 1);
+
+	      if (charcount == G_N_ELEMENTS (xcharbuffer) ||
+		  (charcount > 0 && (glyph_y != glyph_y0 ||
+				     glyph_x != expected_x)))
+		FLUSH;
+
+	      if (charcount == 0)
+		{
+		  glyph_x0 = glyph_x;
+		  glyph_y0 = glyph_y;
+		}
+	      xcharbuffer[charcount].byte1 = index / 256;
+	      xcharbuffer[charcount].byte2 = index % 256;
+
+	      expected_x = glyph_x + XTextWidth16 (fs, &xcharbuffer[charcount], 1);
+
+	      charcount++;
 	    }
 	}
 
     next_glyph:
       x_off += glyphs->glyphs[i].geometry.width;
     }
+  FLUSH;
 }
 
+#undef FLUSH
+
 static void
 pango_x_font_get_glyph_extents  (PangoFont      *font,
 				 PangoGlyph      glyph,
@@ -1645,72 +1678,32 @@ pango_x_render_layout (Display         *display,
 		       int              x, 
 		       int              y)
 {
-  PangoRectangle logical_rect;
-  GSList *tmp_list;
-  PangoAlignment align;
-  int indent;
-  int width;
-  int y_offset = 0;
+  PangoLayoutIter *iter;
 
-  gboolean first = TRUE;
-  
   g_return_if_fail (display != NULL);
-  g_return_if_fail (layout != NULL);
+  g_return_if_fail (PANGO_IS_LAYOUT (layout));
 
-  indent = pango_layout_get_indent (layout);
-  width = pango_layout_get_width (layout);
-  align = pango_layout_get_alignment (layout);
+  iter = pango_layout_get_iter (layout);
 
-  if (width == -1 && align != PANGO_ALIGN_LEFT)
+  do
     {
-      pango_layout_get_extents (layout, NULL, &logical_rect);
-      width = logical_rect.width;
-    }
-  
-  tmp_list = pango_layout_get_lines (layout);
-  while (tmp_list)
-    {
-      PangoLayoutLine *line = tmp_list->data;
-      int x_offset;
+      PangoRectangle   logical_rect;
+      PangoLayoutLine *line;
+      int              baseline;
+      
+      line = pango_layout_iter_get_line (iter);
+      
+      pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
+      baseline = pango_layout_iter_get_baseline (iter);
       
-      pango_layout_line_get_extents (line, NULL, &logical_rect);
-
-      if (width != 1 && align == PANGO_ALIGN_RIGHT)
-	x_offset = width - logical_rect.width;
-      else if (width != 1 && align == PANGO_ALIGN_CENTER)
-	x_offset = (width - logical_rect.width) / 2;
-      else
-	x_offset = 0;
-
-      if (first)
-	{
-	  if (indent > 0)
-	    {
-	      if (align == PANGO_ALIGN_LEFT)
-		x_offset += indent;
-	      else
-		x_offset -= indent;
-	    }
-
-	  first = FALSE;
-	}
-      else
-	{
-	  if (indent < 0)
-	    {
-	      if (align == PANGO_ALIGN_LEFT)
-		x_offset -= indent;
-	      else
-		x_offset += indent;
-	    }
-	}
-	  
       pango_x_render_layout_line (display, drawable, gc,
-				  line, x + x_offset / PANGO_SCALE, y + (y_offset - logical_rect.y) / PANGO_SCALE);
-
-      y_offset += logical_rect.height;
-      tmp_list = tmp_list->next;
+                                  line,
+                                  x + PANGO_PIXELS (logical_rect.x),
+                                  y + PANGO_PIXELS (baseline));
     }
+  while (pango_layout_iter_next_line (iter));
+
+  pango_layout_iter_free (iter);
 }
 
 /* This utility function is duplicated here and in pango-layout.c; should it be
