@@ -33,7 +33,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <stdio.h>
+#include <glib/gprintf.h>
 
 #if USE_LA_MODULES
 #define SOEXT ".la"
@@ -42,6 +42,48 @@
 #endif
 #define SOEXT_LEN (strlen (SOEXT))
 
+static gboolean
+string_needs_escape (const char *str)
+{
+  while (TRUE)
+    {
+      char c = *str++;
+
+      if (!c)
+	return FALSE;
+      else if (c == '\"' || g_ascii_isspace (c))
+	return TRUE;
+    }
+}
+
+static char *
+escape_string (const char *str)
+{
+  GString *result = g_string_new ("");
+
+  while (TRUE)
+    {
+      char c = *str++;
+      
+      switch (c)
+	{
+	case '\0':
+	  goto done;
+	case '\n':
+	  g_string_append (result, "\\n");
+	  break;
+	case '\"':
+	  g_string_append (result, "\\\"");
+	  break;
+	default:
+	  g_string_append_c (result, c);
+	}
+    }
+
+ done:
+  return g_string_free (result, FALSE);
+}
+
 void 
 query_module (const char *dir, const char *name)
 {
@@ -60,7 +102,7 @@ query_module (const char *dir, const char *name)
   module = g_module_open (path, 0);
 
   if (!module)
-    fprintf(stderr, "Cannot load module %s: %s\n", path, g_module_error());
+    g_printerr ("Cannot load module %s: %s\n", path, g_module_error ());
 	  
   if (module &&
       g_module_symbol (module, "script_engine_list", (gpointer *) &list) &&
@@ -78,10 +120,10 @@ query_module (const char *dir, const char *name)
 	  const gchar *quote;
 	  gchar *quoted_path;
 
-	  if (strchr (path, ' ') != NULL)
+	  if (string_needs_escape (path))
 	    {
 	      quote = "\"";
-	      quoted_path = g_strescape (path, NULL);
+	      quoted_path = escape_string (path);
 	    }
 	  else
 	    {
@@ -89,25 +131,25 @@ query_module (const char *dir, const char *name)
 	      quoted_path = g_strdup (path);
 	    }
 	  
-	  printf ("%s%s%s %s %s %s ", quote, quoted_path, quote,
-		  engines[i].id, engines[i].engine_type, engines[i].render_type);
+	  g_printf ("%s%s%s %s %s %s ", quote, quoted_path, quote,
+		    engines[i].id, engines[i].engine_type, engines[i].render_type);
 	  g_free (quoted_path);
 
 	  for (j=0; j < engines[i].n_ranges; j++)
 	    {
 	      if (j != 0)
-		printf (" ");
-	      printf ("%d-%d:%s",
+		g_printf (" ");
+	      g_printf ("%d-%d:%s",
 		      engines[i].ranges[j].start,
 		      engines[i].ranges[j].end,
 		      engines[i].ranges[j].langs);
 	    }
-	  printf ("\n");
+	  g_printf ("\n");
 	}
     }
   else
     {
-      fprintf (stderr, "%s does not export Pango module API\n", path);
+      g_printerr ("%s does not export Pango module API\n", path);
     }
   
   g_free (path);
@@ -121,9 +163,9 @@ int main (int argc, char **argv)
   int i;
   char *path;
 
-  printf ("# Pango Modules file\n"
-	  "# Automatically generated file, do not edit\n"
-	  "#\n");
+  g_printf ("# Pango Modules file\n"
+	    "# Automatically generated file, do not edit\n"
+	    "#\n");
 
   if (argc == 1)		/* No arguments given */
     {
@@ -137,7 +179,7 @@ int main (int argc, char **argv)
 				 "modules",
 				 NULL);
 
-      printf ("# ModulesPath = %s\n#\n", path);
+      g_printf ("# ModulesPath = %s\n#\n", path);
 
       dirs = pango_split_file_list (path);
 
